
#include "navierstokes_bcs.h"
#include <fstream>
#include <iostream>


InletVelocity::InletVelocity()
        : Function<dim>(dim)
{}

void InletVelocity::vector_value(const Point<dim> & p,
                          Vector<double> &values) const
{

    if(inlet_type == "parabolic")
    {
        if(dim==2)
        {
/*
            if(time < plateau_time)
                values(0) =  (time/plateau_time)*(4. * inlet_velocity_x * p(1) * (H - p(1)) / (H * H));
            else
                values(0) = 4. * inlet_velocity_x * p(1) * (H - p(1)) / (H * H);
                */

            values(0) = 4. * inlet_velocity_x * p(1) * (H - p(1)) / (H * H);
            values(1) = inlet_velocity_y;
        }

    }
    else
    {
        if(dim==2)
        {
            //values(0) = inlet_velocity_x + sin(2*M_PI*0.1*time);
            //values(0) = 2*(1+inlet_velocity_x * sin(2*M_PI*time))/3;
            values(0) = inlet_velocity_x;
            values(1) = inlet_velocity_y;
        }
        else
        {

            double vertical_angle = channels_angle * M_PI /180;
            double radius = abs(r2-r1);
            double dx_center = radius/2 + r1;

            double dx21 = dx_center;
            double dx31 = p(0);
            double dy21 = 0.0;
            double dy31 = p(1);
            double m12 = sqrt( dx21*dx21 + dy21*dy21 );
            double m13 = sqrt( dx31*dx31 + dy31*dy31 );
            double theta = acos( (dx21*dx31  + dy21*dy31) / (m12 * m13) );

            double orizzontal_velocity = std::abs(inlet_velocity_z * sin(vertical_angle));

            if(p(0)>= 0 && p(1)>= 0)
            {
                values(0) = - (std::abs(orizzontal_velocity * cos(theta)));
                values(1) = - (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(p(0)>= 0 && p(1)< 0)
            {
                values(0) = - (std::abs(orizzontal_velocity * cos(theta)));
                values(1) =   (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(p(0)< 0 && p(1)>= 0)
            {
                values(0) =   (std::abs(orizzontal_velocity * cos(theta)));
                values(1) = - (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(p(0)< 0 && p(1)< 0)
            {
                values(0) = std::abs(orizzontal_velocity * cos(theta));
                values(1) = std::abs(orizzontal_velocity * sin(theta));
            }

/*
            values(0) = orizzontal_velocity * cos(theta);
            values(1) = orizzontal_velocity * sin(theta);
            */
            values(2) = - (std::abs(inlet_velocity_z * cos(vertical_angle)));

        }
    }

}

void InletVelocity::vector_value_list(const std::vector<Point<dim>> &points,
                  std::vector<Vector<double>> &  value_list) const
{
    Assert(value_list.size() == points.size(),
       ExcDimensionMismatch(value_list.size(), points.size()));
    for (unsigned int p = 0; p < points.size(); ++p)
    InletVelocity::vector_value(points[p], value_list[p]);
}





InletVelocityLaser::InletVelocityLaser()
        : Function<dim>(dim)
{}

void InletVelocityLaser::vector_value(const Point<dim> & p,
                                 Vector<double> &values) const
{

    values(0) = 0.0;
    values(1) = 0.0;
    values(2) = 0.0;

}

void InletVelocityLaser::vector_value_list(const std::vector<Point<dim>> &points,
                                      std::vector<Vector<double>> &  value_list) const
{
    Assert(value_list.size() == points.size(),
           ExcDimensionMismatch(value_list.size(), points.size()));
    for (unsigned int p = 0; p < points.size(); ++p)
        InletVelocityLaser::vector_value(points[p], value_list[p]);
}





Pressure::Pressure()
        : Function<dim>(1)
{}

double Pressure::value(const Point<dim> & p,
                       const unsigned int component) const
{
    (void)component;
    AssertIndexRange(component, 1);
    return (l_channel - p(2));
}

void Pressure::value_list(const std::vector<Point<dim>> &points,
                          std::vector<double> &          values,
                          const unsigned int component) const
{
    (void)component;
    AssertIndexRange(component, 1);
    const unsigned int n_points = points.size();
    Assert(values.size() == n_points,
           ExcDimensionMismatch(values.size(), n_points));
    for (unsigned int i = 0; i < n_points; ++i)
        values[i] = Pressure::value(points[i]);
}


InitialVelocity::InitialVelocity()
        : Function<dim>(2)
{}

void InitialVelocity::vector_value(const Point<dim> & p,
                                 Vector<double> &values) const
{

    values(0) = 0.0;
    values(1) = 0.0;
    values(2) = 0.0;


}

void InitialVelocity::vector_value_list(const std::vector<Point<dim>> &points,
                                      std::vector<Vector<double>> &  value_list) const
{
    Assert(value_list.size() == points.size(),
           ExcDimensionMismatch(value_list.size(), points.size()));
    for (unsigned int p = 0; p < points.size(); ++p)
        InitialVelocity::vector_value(points[p], value_list[p]);
}