
#ifndef MULTIGRID_BCS_H
#define MULTIGRID_BCS_H

#include <deal.II/base/function.h>
#include <deal.II/lac/vector.h>

#include "../general.h"

using namespace dealii;

class InletVelocity : public Function<dim>
{
public:

    double inlet_velocity_x, inlet_velocity_y, inlet_velocity_z, H;
    double time, plateau_time, channels_angle, r1, r2;
    std::string inlet_type;

    InletVelocity();

    virtual void vector_value(const Point<dim> & p,
                              Vector<double> &values) const override;

    virtual void
    vector_value_list(const std::vector<Point<dim>> &points,
                      std::vector<Vector<double>> &  value_list) const override;
};

class InletVelocityLaser : public Function<dim>
{
public:

    double inlet_velocity_x, inlet_velocity_y, inlet_velocity_z, H;
    double time, plateau_time;
    std::string inlet_type;

    InletVelocityLaser();

    virtual void vector_value(const Point<dim> & p,
                              Vector<double> &values) const override;

    virtual void
    vector_value_list(const std::vector<Point<dim>> &points,
                      std::vector<Vector<double>> &  value_list) const override;
};



class Pressure : public Function<dim>
{
public:

    double l_channel;

    Pressure();

    virtual double value(const Point<dim> & p,
                         const unsigned int component = 0) const override;

    virtual void value_list(const std::vector<Point<dim>> &points,
                            std::vector<double> &          values,
                            const unsigned int component = 0) const override;
};



class InitialVelocity : public Function<dim>
{
public:


    InitialVelocity();

    virtual void vector_value(const Point<dim> & p,
                              Vector<double> &values) const override;

    virtual void
    vector_value_list(const std::vector<Point<dim>> &points,
                      std::vector<Vector<double>> &  value_list) const override;
};




#endif //MULTIGRID_BCS_H
