
#ifndef MULTIGRID_WORKER_H
#define MULTIGRID_WORKER_H

#include <deal.II/fe/fe_q.h>
#include <deal.II/fe/mapping_q.h>
#include <deal.II/fe/fe_values.h>

#include <deal.II/lac/vector.h>
#include <deal.II/lac/full_matrix.h>

#include "../general.h"
#include "navierstokes.h"

using namespace dealii;

class NS_ScratchData
{
public:

    NS_ScratchData(const NavierStokes &ns,
                const UpdateFlags         update_flags = update_values |
                                                         update_gradients |
                                                         update_hessians |
                                                         update_quadrature_points |
                                                         update_JxW_values);

    NS_ScratchData(const NS_ScratchData &scratch_data);

    const NavierStokes &ns;
    UpdateFlags update_scratch;
    FEValues<dim> fe_values_velocity;
    FEValues<dim> fe_values_pressure;
    FEValues<dim> fe_values_pressure_grad;


};


class NS_CopyData
{
public:
    NS_CopyData(const NavierStokes &ns)
    : ns(&ns)
    {};

    int level;
    int dof_per_cell_velocity;
    int dof_per_cell_pressure;

    FullMatrix<double>                   cell_matrix;
    Vector<double>                       cell_rhs;
    std::vector<types::global_dof_index> local_dof_indices_velocity;
    std::vector<types::global_dof_index> local_dof_indices_pressure;


    const NavierStokes *ns;


};


#endif //MULTIGRID_WORKER_H
