

#include "mesher.h"


mesher::mesher(const Settings &settings)
: settings(settings)
{}

void mesher::rectangle_with_hole(Triangulation<dim> &triangulation)
{
    GridGenerator::hyper_cube_with_cylindrical_hole(triangulation, 0.3, 1.0);

    // Set manifold for the inner (curved) boundary.
    static const SphericalManifold<dim> manifold_description(Point<dim>(0, 0));
    triangulation.set_manifold(1, manifold_description);

}

void mesher::cylinder(Triangulation<dim> &triangulation) {

    GridGenerator::cylinder(triangulation, 1.0, 10.0);
}

void mesher::rectangle(Triangulation<dim> &triangulation)
{
    GridGenerator::hyper_cube(triangulation, 0, 1);
    //GridGenerator::hyper_rectangle(triangulation, Point<dim> (0, 0), Point<dim> (1, 1));

}

void mesher::channel_with_cylinder(Triangulation<dim> &triangulation)
{
    GridGenerator::channel_with_cylinder(triangulation,
                                         /*const double shell_region_width =*/ 0.03,
                                         /*const unsigned int n_shells =*/ 2,
                                         /*const double skewness =*/ 2.0,
                                         /*const bool colorize =*/ true);
}

void mesher::load_abaqus_mesh(Triangulation<dim> &triangulation)
{
    GridIn<dim> grid_in;
    grid_in.attach_triangulation(triangulation);
    std::string filename = settings.mesh_file;
    std::ifstream file(filename);
    Assert(file, ExcFileNotOpen(filename.c_str()));
    grid_in.read_ucd(file);
    //grid_in.read_abaqus(file, true);

    std::map<int, int> material_map;

    typename Triangulation<dim>::active_cell_iterator
            cell = triangulation.begin_active(),
            endc = triangulation.end();
    for (; cell!=endc; ++cell)
    {
      material_map[cell->material_id()]++;
    }

    for(auto p:material_map)
    {
        std::cout << "primo matrial id: " << p.first << "secondo matrial id: " << p.second << std::endl;
    }





}

void mesher::load_gmsh_mesh(Triangulation<dim> &triangulation)
{

    GridIn<dim> grid_in;
    grid_in.attach_triangulation(triangulation);
    std::string filename = settings.mesh_file;
    std::ifstream file(filename);
    grid_in.read_msh(file);

}

void mesher::load_salome_mesh(Triangulation<dim> &triangulation)
{

    GridIn<dim> grid_in;
    grid_in.attach_triangulation(triangulation);
    std::string filename = settings.mesh_file;
    std::ifstream file(filename);
    grid_in.read_unv(file);

    deallog << "MESH PARAMETERS " << std::endl;
    deallog << "filename:  " << filename << std::endl;



    for (const auto &cell : triangulation.cell_iterators())
        for (unsigned int face_number = 0; face_number < GeometryInfo<dim>::faces_per_cell; ++face_number)
        {
            if (cell->face(face_number)->at_boundary())
                cell->face(face_number)->set_boundary_id(settings.walls_manifold);
        }


    // Parameters simulation

    for (const auto &cell : triangulation.cell_iterators())
        for (unsigned int face_number = 0; face_number < GeometryInfo<dim>::faces_per_cell; ++face_number)
        {
            if(cell->face(face_number)->at_boundary())
            {
                const auto center = cell->face(face_number)->center();
                double radius = sqrt(pow(std::fabs(center(0)),2) + pow(std::fabs(center(1)),2));

                if(center(2) < settings.substrate_location + 1e-12)
                {
                    cell->face(face_number)->set_boundary_id(settings.substrate_manifold);
                }
                /*
                if(center(2) > 0.015 - 1e-12 && radius < 0.005)
                {
                    cell->face(face_number)->set_boundary_id(settings.laser_manifold);
                }
                 */
                if(center(2) > settings.top_point_mesh - 1e-12 /* && radius > 0.005 */ )
                {
                    cell->face(face_number)->set_boundary_id(settings.inlet_manifold);
                }
                if(radius > settings.box_radius - 1e-03)
                {
                    cell->face(face_number)->set_boundary_id(settings.outlet_manifold);
                }

            }
        }


/*
    // Mesh without substrate

    for (const auto &cell : triangulation.cell_iterators())
        for (unsigned int face_number = 0; face_number < GeometryInfo<dim>::faces_per_cell; ++face_number)
        {
            if(cell->face(face_number)->at_boundary())
            {
                const auto center = cell->face(face_number)->center();
                double radius = sqrt(pow(std::fabs(center(0)),2) + pow(std::fabs(center(1)),2));

                if(center(2) < settings.substrate_location + 1e-12)
                {
                    cell->face(face_number)->set_boundary_id(settings.outlet_manifold);
                }

                if(center(2) > 0.015 - 1e-12 && radius < 0.005)
                {
                    cell->face(face_number)->set_boundary_id(settings.laser_manifold);
                }

                if(center(2) > 0.015 - 1e-12 && radius > 0.005)
                {
                    cell->face(face_number)->set_boundary_id(settings.inlet_manifold);
                }
            }
        }


    // Mesh with substrate

    for (const auto &cell : triangulation.cell_iterators())
        for (unsigned int face_number = 0; face_number < GeometryInfo<dim>::faces_per_cell; ++face_number)
        {
            if(cell->face(face_number)->at_boundary())
            {
                const auto center = cell->face(face_number)->center();
                double radius = sqrt(pow(std::fabs(center(0)),2) + pow(std::fabs(center(1)),2));

                if(center(2) < settings.substrate_location + 1e-12)
                {
                    cell->face(face_number)->set_boundary_id(settings.substrate_manifold);
                }
                if(center(2) > 0.015 - 1e-12 && radius < 0.005)
                {
                    cell->face(face_number)->set_boundary_id(settings.laser_manifold);
                }
                if(center(2) > 0.015 - 1e-12 && radius > 0.005)
                {
                    cell->face(face_number)->set_boundary_id(settings.inlet_manifold);
                }
                if(radius > 0.015 - 1e-03)
                {
                    cell->face(face_number)->set_boundary_id(settings.outlet_manifold);
                }

            }
        }

*/

    //-- Boundary count
    std::map<unsigned int, unsigned int> boundary_count;
    typename Triangulation<dim>::active_cell_iterator
            cell = triangulation.begin_active(),
            endc = triangulation.end();
    for (; cell!=endc; ++cell)
    {
        for (unsigned int face=0; face<GeometryInfo<dim>::faces_per_cell; ++face)
        {
            if (cell->face(face)->at_boundary())
                boundary_count[cell->face(face)->boundary_id()]++;
        }
    }
    deallog << " boundary indicators: ";
    for (std::map<unsigned int, unsigned int>::iterator it=boundary_count.begin();
         it!=boundary_count.end(); ++it)
    {
        deallog << it->first << "(" << it->second << " times) ";
    }
    deallog << std::endl;
}



void mesher::boundary_refine(Triangulation<dim> &triangulation)
{
    std::vector<types::boundary_id> boundary_ids;
    boundary_ids = triangulation.get_boundary_ids();

    {
        std::map<types::boundary_id, unsigned int> boundary_count;

        for (auto &cell : triangulation.active_cell_iterators())
        {
            for (unsigned int face = 0; face < GeometryInfo<dim>::faces_per_cell;
                 ++face)
            {
                if (cell->face(face)->at_boundary())
                {
                    boundary_count[cell->face(face)->boundary_id()]++;
                    if((int) cell->face(face)->boundary_id() == settings.walls_manifold
                    || (int) cell->face(face)->boundary_id() == settings.laser_manifold)
                        cell->set_refine_flag();
                }
            }


        }
        triangulation.execute_coarsening_and_refinement();

        /*
        std::cout << " boundary indicators: ";
        for (const std::pair<const types::boundary_id, unsigned int> &pair :
                boundary_count)
        {
            std::cout << pair.first << "(" << pair.second << " times) ";
        }
        std::cout << std::endl;
         */
    }
}

void mesher::vertex_refine(Triangulation<dim> &triangulation)
{
    Triangulation<dim, dim>::active_cell_iterator cell = triangulation.begin_active(), endc = triangulation.end();

    for (; cell != endc; ++cell)
    {
        cell->clear_coarsen_flag();
        for (unsigned int v = 0; v < GeometryInfo<2>::vertices_per_cell; ++v)
        {
            if(cell->vertex(v)[1] > 9.75)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 5.25 && cell->vertex(v)[0] < 10.24)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 0.24)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 6.49 && cell->vertex(v)[1] > 3.51
               && cell->vertex(v)[0] < 15.24 && cell->vertex(v)[0] > 12.26 )
                cell->set_refine_flag();
        }
    }

    /*
    for (auto &cell : triangulation.active_cell_iterators())
    {
        cell->clear_coarsen_flag();

        for (unsigned int v = 0; v < GeometryInfo<2>::vertices_per_cell; ++v)
        {
            if(cell->vertex(v)[1] > 9.75)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 5.25 && cell->vertex(v)[0] < 10.24)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 0.24)
                cell->set_refine_flag();
            if(cell->vertex(v)[1] < 6.49 && cell->vertex(v)[1] > 3.51
               && cell->vertex(v)[0] < 15.24 && cell->vertex(v)[0] > 12.26 )
                cell->set_refine_flag();
        }
    }
     */
    triangulation.execute_coarsening_and_refinement();
}