#ifndef NS_NEWTON_NEWTON_H
#define NS_NEWTON_NEWTON_H


#include <deal.II/base/quadrature_lib.h>
#include <deal.II/base/function.h>
#include <deal.II/base/utilities.h>
#include <deal.II/base/tensor.h>

#include <deal.II/lac/block_vector.h>
#include <deal.II/lac/full_matrix.h>
#include <deal.II/lac/block_sparse_matrix.h>
#include <deal.II/lac/dynamic_sparsity_pattern.h>
#include <deal.II/lac/solver_cg.h>
#include <deal.II/lac/solver_gmres.h>
#include <deal.II/lac/precondition.h>
#include <deal.II/lac/affine_constraints.h>
#include <deal.II/lac/sparse_direct.h>

#include <deal.II/grid/tria.h>
#include <deal.II/grid/grid_generator.h>
#include <deal.II/grid/grid_refinement.h>
#include <deal.II/grid/grid_tools.h>
#include <deal.II/grid/grid_in.h>

#include <deal.II/dofs/dof_handler.h>
#include <deal.II/dofs/dof_renumbering.h>
#include <deal.II/dofs/dof_tools.h>

#include <deal.II/fe/fe_q.h>
#include <deal.II/fe/fe_system.h>
#include <deal.II/fe/fe_values.h>

#include <deal.II/numerics/vector_tools.h>
#include <deal.II/numerics/matrix_tools.h>
#include <deal.II/numerics/data_out.h>
#include <deal.II/numerics/error_estimator.h>

// To transfer solutions between meshes, this file is included:
#include <deal.II/numerics/solution_transfer.h>

// This file includes UMFPACK: the direct solver:
#include <deal.II/lac/sparse_direct.h>

// And the one for ILU preconditioner:
#include <deal.II/lac/sparse_ilu.h>


#include <fstream>
#include <iostream>

#include "../general.h"
#include "newton_bcs.h"
#include "../settings.h"


using namespace dealii;

class NavierStokesNewton
{
public:
    NavierStokesNewton(const Settings &settings, Triangulation<dim> &triangulation);

    void setup_dofs();

    void initialize_system();

    void assemble(const bool initial_step, const bool assemble_matrix);

    void assemble_system(const bool initial_step);

    void assemble_rhs(const bool initial_step);

    void solve(const bool initial_step);

    void output_results(DataOut<dim> &data_out) const;


    void direct_newton_iteration(const double       tolerance,
                          const unsigned int max_iter);

    void direct_quasi_newton_iteration(const double       tolerance,
                                 const unsigned int max_iter);

    double initialize_solution(bool is_initial_step);

    void compute_initial_guess(bool small_reynolds, double step_size = 1.0);

    void update_boundary_values(double time);

    void compute_errors();

    double                               viscosity;
    double                               gamma;
    const unsigned int                   degree;
    std::vector<types::global_dof_index> dofs_per_block;

    Triangulation<dim> &triangulation;
    FESystem<dim>      fe;
    DoFHandler<dim>    dof_handler;

    AffineConstraints<double> zero_constraints;
    AffineConstraints<double> nonzero_constraints;

    BlockSparsityPattern      sparsity_pattern;
    BlockSparseMatrix<double> system_matrix;
    SparseMatrix<double>      pressure_mass_matrix;

    BlockVector<double> present_solution;
    BlockVector<double> newton_update;
    BlockVector<double> system_rhs;
    BlockVector<double> evaluation_point;

    BlockVector<double> evaluation_point_minus_1;
    BlockVector<double> present_solution_minus_1;

    const double time_step;

    SparseDirectUMFPACK A_direct;

    const Settings &settings;

    InletVelocityNewton inlet_velocity_boundary;

    bool init_step;
    bool update_boundary;
    double res_after_initial_guess;

    double l2_norm_div_u;
};


#endif //NS_NEWTON_NEWTON_H
