#include "newton.h"



void NavierStokesNewton::direct_newton_iteration(
        const double       tolerance,
        const unsigned int max_iter)
{
//    for (unsigned int refinement_n = 0; refinement_n < max_n_refinements + 1; ++refinement_n)

    unsigned int n_iter = 0;
    double       last_res      = 1.0;
    double       current_res   = 1.0;


    if(init_step == true)
    {
        last_res = res_after_initial_guess;
        init_step = false;
    }


    /*
    if(settings.time_dependent_buondary == true)
    {
        update_boundary = true;
    }
     */


    while ( (current_res > tolerance) && n_iter < max_iter) {

        evaluation_point = present_solution;

        statistics->enter_subsection("Newton Assembly");
        assemble_system(false);
        statistics->leave_subsection();

        statistics->enter_subsection("Newton Factorize");
        A_direct.factorize(system_matrix);
        statistics->leave_subsection();

        statistics->enter_subsection("Newton Solve");
        solve(false);
        statistics->leave_subsection();


        for (double alpha = 1.0; alpha > 1e-5; alpha *= 0.5) {

            evaluation_point = present_solution;
            evaluation_point.add(alpha, newton_update);
            nonzero_constraints.distribute(evaluation_point);


            statistics->enter_subsection("Newton Assembly RHS");
            assemble_rhs(false);
            statistics->leave_subsection();

            current_res = system_rhs.l2_norm();
            deallog << "  alpha: " << std::setw(10) << alpha
                    << std::setw(0) << "  residual: " << current_res
                    << std::endl;

            if (current_res < last_res)
                break;
        }

        present_solution = evaluation_point;

        deallog << "  number of line searches: " << n_iter
                << "  residual: " << current_res << std::endl;
        last_res = current_res;
        ++n_iter;

        if (n_iter == max_iter)
            deallog << "  Newton NOT CONVERGED " << std::endl;

    }

    evaluation_point_minus_1 = evaluation_point;
    present_solution_minus_1 = evaluation_point_minus_1;

    compute_errors();
}


void NavierStokesNewton::direct_quasi_newton_iteration(
        const double       tolerance,
        const unsigned int max_iter)
{
//    for (unsigned int refinement_n = 0; refinement_n < max_n_refinements + 1; ++refinement_n)

    unsigned int n_iter = 0;
    double       last_res      = 1.0;
    double       current_res   = 1.0;

    if(init_step == true)
    {
        last_res = res_after_initial_guess;
        init_step = false;
    }

    if(settings.time_dependent_buondary == true)
    {
        update_boundary = true;
    }

    evaluation_point = present_solution;

    statistics->enter_subsection("Newton Assembly");
    assemble_system(update_boundary);
    statistics->leave_subsection();

    statistics->enter_subsection("Newton Factorize");
    A_direct.factorize(system_matrix);
    statistics->leave_subsection();

    while ( (current_res > tolerance) && n_iter < max_iter)
    {
        statistics->enter_subsection("Newton Solve");
        solve(update_boundary);
        statistics->leave_subsection();

        for (double alpha = 1.0; alpha > 1e-5; alpha *= 0.5)
        {
            evaluation_point = present_solution;
            evaluation_point.add(alpha, newton_update);
            nonzero_constraints.distribute(evaluation_point);

            statistics->enter_subsection("Newton Assembly RHS");
            assemble_rhs(false);
            statistics->leave_subsection();

            current_res = system_rhs.l2_norm();
            deallog << "  alpha: " << std::setw(10) << alpha
                    << std::setw(0) << "  residual: " << current_res
                    << std::endl;

            if (current_res < last_res)
                break;
        }
        present_solution = evaluation_point;
        deallog << "  number of line searches: " << n_iter
                << "  residual: " << current_res << std::endl;
        last_res = current_res;
        ++n_iter;

        if(n_iter == max_iter)
            deallog << "  Newton NOT CONVERGED " << std::endl;


    }

    evaluation_point_minus_1 = evaluation_point;
    present_solution_minus_1 = evaluation_point_minus_1;

    compute_errors();
}

