#include "newton.h"





void NavierStokesNewton::solve(const bool initial_step)
{
    newton_update = system_rhs;
    A_direct.solve(newton_update);
    const AffineConstraints<double> &constraints_used = initial_step ? nonzero_constraints : zero_constraints;
    constraints_used.distribute(newton_update);
}


double NavierStokesNewton::initialize_solution(bool is_initial_step)
{
    evaluation_point = present_solution;

    statistics->enter_subsection("Newton Assembly");
    assemble_system(is_initial_step);
    statistics->leave_subsection();

    statistics->enter_subsection("Newton Factorize");
    A_direct.factorize(system_matrix);
    statistics->leave_subsection();

    statistics->enter_subsection("Newton Solve");
    solve(is_initial_step);
    statistics->leave_subsection();

    present_solution = newton_update;
    nonzero_constraints.distribute(present_solution);
    evaluation_point = present_solution;

    statistics->enter_subsection("Newton Assembly RHS");
    assemble_rhs(false);
    statistics->leave_subsection();

    return system_rhs.l2_norm();

}

void NavierStokesNewton::compute_initial_guess(bool small_reynolds, double step_size)
{
    init_step = true;
    double current_res;

    if( small_reynolds )
    {
        current_res = initialize_solution(true);
        deallog << "The residual of initial guess is " << current_res << std::endl;
        res_after_initial_guess = current_res;
        //if(settings.algorithm == "quasi_newton")
          //  direct_newton_iteration(settings.newton_tolerance, settings.max_newton_iterations);
    }
    else
    {
        const double target_Re = 1.0 / viscosity;

        viscosity = 1.0 / 1000;
        deallog << "Searching for initial guess with Re = 1000" << std::endl;
        current_res = initialize_solution(true);
        deallog << "The residual of initial guess is " << current_res << std::endl;
        res_after_initial_guess = current_res;

        if(settings.algorithm == "newton")
            direct_newton_iteration(settings.newton_tolerance, settings.max_newton_iterations);

        if(settings.algorithm == "quasi_newton")
            direct_newton_iteration(settings.newton_tolerance, settings.max_newton_iterations);


        for (double Re = 1000.0 + step_size; Re < target_Re;
             Re        = std::min(Re + step_size, target_Re))
        {
            viscosity = 1.0 / Re;
            deallog << "Searching for initial guess with Re = " << Re << std::endl;

            if(settings.algorithm == "newton")
                direct_newton_iteration(settings.newton_tolerance, settings.max_newton_iterations);

            if(settings.algorithm == "quasi_newton")
                direct_newton_iteration(settings.newton_tolerance, settings.max_newton_iterations);

        }
    }
}

