#include "particles_dynamic.h"


// FIXED RATE OF PARTICLES
void ParticlesDynamic::generate_particles()
{

    if (next_slot_index < inlet_slots)
    {
        generate_particles_on_ring(n_particles_per_slot, settings.inner_radius_coaxial_nozzle, /*0.008802,*/
                                                         settings.outer_radius_coaxial_nozzle /*0.0099*/);
        //generate_particles_on_boundary(n_particles_per_cell_per_slot);
        ++next_slot_index;
    }
}

void ParticlesDynamic::generate_particles_on_boundary(unsigned int particles_to_generate) {


    for (auto &f:inlet_cells) {
        Triangulation<dim>::active_cell_iterator inlet_cell_iterator(&triangulation, f.first, f.second);

        const auto center = inlet_cell_iterator->center();
        auto cell_diameter = inlet_cell_iterator->diameter();

        for (int i = 0; i < particles_to_generate; i++) {

            double random_x = -1. + (rand() / (RAND_MAX / (2.)));
            double random_y = -1. + (rand() / (RAND_MAX / (2.)));
            double random_z = -1. + (rand() / (RAND_MAX / (2.)));

            const Point<dim> initial_position = center; //mapping.transform_unit_to_real_cell(cell, center);

            Point<dim> random_vector((cell_diameter / 3) * random_x, (cell_diameter / 3) * random_y, (cell_diameter / 3) * random_z);
            //Point<dim> random_vector(0, (cell_diameter / 2) * random_y);

            auto adjusted_position_random_real = initial_position + random_vector;

            try {
                auto is_in_cell = GridTools::find_active_cell_around_point(triangulation,
                                                                           adjusted_position_random_real);
            }
            catch (...) {
                adjusted_position_random_real = initial_position;
            }

            auto ref_cell = GridTools::find_active_cell_around_point(dof_handler_temperature, adjusted_position_random_real);
            Point<dim> ref_p = StaticMappingQ1<dim>::mapping.transform_real_to_unit_cell(ref_cell, adjusted_position_random_real);


            double vertical_angle = settings.channel_angle * M_PI /180;
            double radius = abs(settings.outer_radius_coaxial_nozzle-settings.inner_radius_coaxial_nozzle);
            double dx_center = radius/2 + settings.inner_radius_coaxial_nozzle;

            double dx21 = dx_center;
            double dx31 = adjusted_position_random_real(0);
            double dy21 = 0.0;
            double dy31 = adjusted_position_random_real(1)-0.0;
            double m12 = sqrt( dx21*dx21 + dy21*dy21 );
            double m13 = sqrt( dx31*dx31 + dy31*dy31 );
            double theta = acos( (dx21*dx31  + dy21*dy31) / (m12 * m13) );

            double orizzontal_velocity = std::abs(initial_particle_velocity[2] * sin(vertical_angle));

            double value_x;
            double value_y;
            double value_z;

            if(adjusted_position_random_real(0)>= 0 && adjusted_position_random_real(1)>= 0)
            {
                value_x = - (std::abs(orizzontal_velocity * cos(theta)));
                value_y = - (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(adjusted_position_random_real(0)>= 0 && adjusted_position_random_real(1)< 0)
            {
                value_x = - (std::abs(orizzontal_velocity * cos(theta)));
                value_y =   (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(adjusted_position_random_real(0)< 0 && adjusted_position_random_real(1)>= 0)
            {
                value_x =   (std::abs(orizzontal_velocity * cos(theta)));
                value_y = - (std::abs(orizzontal_velocity * sin(theta)));
            }
            if(adjusted_position_random_real(0)< 0 && adjusted_position_random_real(1)< 0)
            {
                value_x = std::abs(orizzontal_velocity * cos(theta));
                value_y = std::abs(orizzontal_velocity * sin(theta));
            }


            value_z = - (std::abs(initial_particle_velocity[2] * cos(vertical_angle)));


            value_x = initial_particle_velocity[0];
            value_y = initial_particle_velocity[1];
            value_z = initial_particle_velocity[2];


            std::vector<double>  property(num_properties);
            property[num_temperature] = initial_particle_temperature;
            property[num_fluid_temperature] = 0.0;
            property[num_velocity_x] = value_x;
            property[num_fluid_velocity_x] = 0.0;
            property[num_velocity_y] = value_y;
            property[num_fluid_velocity_y] = 0.0;
            property[num_velocity_z] = 0.0;
            property[num_fluid_velocity_z] = 0.0;
            property[num_laser_power] = 0.0;
            property[num_liquid_fraction] = 0.0;
            property[type_ignore] = 0.0;
            property[type_advect] = 0.0;
            property[type_block] = 0.0;

            if(dim > 2)
                property[num_velocity_z] = value_z;


            auto it =  particle_handler.insert_particle(Particles::Particle<dim>(adjusted_position_random_real,
                                                                                 ref_p,
                                                                                 next_particle_index),
                                                                                 inlet_cell_iterator);

            it->set_properties(property);

            ++next_particle_index;


        }
    }
}





void ParticlesDynamic::generate_one_particle(Point<dim> &p) {


    auto ref_cell = GridTools::find_active_cell_around_point(dof_handler_temperature, p);

    Point<dim> ref_p = StaticMappingQ1<dim>::mapping.transform_real_to_unit_cell(ref_cell, p);

    Particles::Particle<dim>* particle = new Particles::Particle<dim>(p, ref_p, next_particle_index);

    std::vector<double>  property(num_properties);
    property[num_temperature] = initial_particle_temperature;
    property[num_fluid_temperature] = 0.0;
    property[num_velocity_x] = initial_particle_velocity[0];
    property[num_fluid_velocity_x] = 0.0;
    property[num_velocity_y] = initial_particle_velocity[1];
    property[num_fluid_velocity_y] = 0.0;
    property[num_velocity_z] = 0.0;
    property[num_fluid_velocity_z] = 0.0;
    property[num_laser_power] = 0.0;
    property[num_liquid_fraction] = 0.0;
    property[type_ignore] = 0.0;
    property[type_advect] = 0.0;
    property[type_block] = 0.0;

    if(dim > 2)
        property[num_velocity_z] = initial_particle_velocity[2];

    particle->set_properties(property);

    auto cell = GridTools::find_active_cell_around_point(dof_handler_temperature, particle->get_location());

    particle_handler.insert_particle(*particle, cell);

    ++next_particle_index;


}






void ParticlesDynamic::generate_particles_on_ring(unsigned int n_particles_per_slot, double r1, double r2) {


    Point<dim> location;

    for (int i = 0; i < n_particles_per_slot; i++)
    {
        double radius = abs(r2-r1);
        double dx_center = radius/2 + r1;

        double random_radius = 0.0 + ((double) rand() / (RAND_MAX/(0.7)));
        double random_theta = ((double) rand() / (RAND_MAX));
        double random_high = ((double) rand() / (RAND_MAX));

        double theta = 2 * M_PI * random_theta;
        double r = radius * sqrt(random_radius);
        double location_x = (r1 + r) * cos(theta);
        double location_y = (r1 + r) * sin(theta);


        location(0) = location_x;
        location(1) = location_y;
        location(2) = settings.top_point_mesh  - (abs(settings.initial_particle_velocity_z)*settings.time_step * random_high)/2;


        double vertical_angle = settings.channel_angle * M_PI /180;

        double dx21 = dx_center;
        double dx31 = location(0);
        double dy21 = 0.0;
        double dy31 = location(1);
        double m12 = sqrt( dx21*dx21 + dy21*dy21 );
        double m13 = sqrt( dx31*dx31 + dy31*dy31 );
        double theta_2 = acos( (dx21*dx31  + dy21*dy31) / (m12 * m13) );

        double orizzontal_velocity = std::abs(initial_particle_velocity[2] * sin(vertical_angle));

        double value_x;
        double value_y;
        double value_z;

        if(location(0)>= 0 && location(1)>= 0)
        {
            value_x = - (std::abs(orizzontal_velocity * cos(theta_2)));
            value_y = - (std::abs(orizzontal_velocity * sin(theta_2)));
        }
        if(location(0)>= 0 && location(1)< 0)
        {
            value_x = - (std::abs(orizzontal_velocity * cos(theta_2)));
            value_y =   (std::abs(orizzontal_velocity * sin(theta_2)));
        }
        if(location(0)< 0 && location(1)>= 0)
        {
            value_x =   (std::abs(orizzontal_velocity * cos(theta_2)));
            value_y = - (std::abs(orizzontal_velocity * sin(theta_2)));
        }
        if(location(0)< 0 && location(1)< 0)
        {
            value_x = std::abs(orizzontal_velocity * cos(theta_2));
            value_y = std::abs(orizzontal_velocity * sin(theta_2));
        }

        value_z = - (std::abs(initial_particle_velocity[2] * cos(vertical_angle)));


        /** Generate diameter with Gaussian distribution **/
        double particle_random_diameter = gaussian_distribution(generate_random_numbers);

        std::vector<double>  property(num_properties);
        property[num_temperature] = initial_particle_temperature;
        property[num_fluid_temperature] = 0.0;
        property[num_velocity_x] = value_x;
        property[num_fluid_velocity_x] = 0.0;
        property[num_velocity_y] = value_y;
        property[num_fluid_velocity_y] = 0.0;
        property[num_velocity_z] = 0.0;
        property[num_fluid_velocity_z] = 0.0;
        property[num_laser_power] = 0.0;
        property[num_liquid_fraction] = 0.0;
        property[num_diameter] = particle_random_diameter;
        property[type_ignore] = -1.0;
        property[type_advect] = 1.0;
        property[type_block] = -1.0;

        if(dim > 2)
            property[num_velocity_z] = value_z;


        try
        {
            auto ref_cell = GridTools::find_active_cell_around_point(dof_handler_temperature, location);
            Point<dim> ref_p = StaticMappingQ1<dim>::mapping.transform_real_to_unit_cell(ref_cell, location);
            auto it =  particle_handler.insert_particle(Particles::Particle<dim>(location, ref_p, next_particle_index),
                                                        ref_cell);
            it->set_properties(property);
            location.clear();
            ++next_particle_index;
        }
        catch (...)
        {
            continue;
        }
    }


}
