
#include "settings.h"

using namespace dealii;

void Settings::get_parameters(const std::string &prm_filename)
{
    ParameterHandler prm;

    prm.enter_subsection("Simulation parameters");
    {
        prm.declare_entry("simulation_time", "10.0", Patterns::Double(0), "simulation_time");
        prm.declare_entry("time_step", "0.005", Patterns::Double(0), "time_step");
        prm.declare_entry("theta", "0.5", Patterns::Double(0), "theta");
        prm.declare_entry("mesh_file","mesh_file", Patterns::Anything(), "Mesh File");
        prm.declare_entry("refinement_mesh_degree", "10", Patterns::Integer(0), "refinement_mesh_degree");
    }
    prm.leave_subsection();

    prm.enter_subsection("FEM parameters");
    {
        prm.declare_entry("fe_degree", "1", Patterns::Integer(1), "Finite Element degree");
    }
    prm.leave_subsection();

    prm.enter_subsection("Physical parameters");
    {
        prm.declare_entry("inlet_manifold","1", Patterns::Integer(0), "inlet_manifold");
        prm.declare_entry("outlet_manifold","2", Patterns::Integer(0), "outlet_manifold");
        prm.declare_entry("walls_manifold","3", Patterns::Integer(0), "walls_manifold");
        prm.declare_entry("laser_manifold","4", Patterns::Integer(0), "laser_manifold");
        prm.declare_entry("substrate_manifold","4", Patterns::Integer(0), "substrate_manifold");
        prm.declare_entry("time_dependent_buondary", "true", Patterns::Bool(), "Enable Time Dependent Buondary: true|false");
        prm.declare_entry("l_channel", "10.0", Patterns::Double(0), "l_channel");
        prm.declare_entry("h_channel", "10.0", Patterns::Double(0), "h_channel");
        prm.declare_entry("substrate_location", "10.0", Patterns::Double(-100), "substrate_location");
        prm.declare_entry("outer_radius_coaxial_nozzle", "10.0", Patterns::Double(-100), "outer_radius_coaxial_nozzle");
        prm.declare_entry("inner_radius_coaxial_nozzle", "10.0", Patterns::Double(-100), "inner_radius_coaxial_nozzle");
        prm.declare_entry("top_point_mesh", "10.0", Patterns::Double(-100), "top_point_mesh");
        prm.declare_entry("channel_angle", "10.0", Patterns::Double(-100), "channel_angle");
        prm.declare_entry("box_radius", "10.0", Patterns::Double(-100), "box_radius");
        prm.declare_entry("inlet_velocity_x", "1.0", Patterns::Double(-100), "inlet_velocity_x");
        prm.declare_entry("inlet_velocity_y", "1.0", Patterns::Double(-100), "inlet_velocity_y");
        prm.declare_entry("inlet_velocity_z", "1.0", Patterns::Double(-100), "inlet_velocity_z");
        prm.declare_entry("inlet_type","constant", Patterns::Anything(), "select: constant | parabolic");
        prm.declare_entry("reynolds_number", "0.005", Patterns::Double(0), "reynolds_number");
        prm.declare_entry("strouhal_number", "0.005", Patterns::Double(0), "strouhal_number");
        prm.declare_entry("prandtl_number", "0.1", Patterns::Double(0), "prandtl_number");
        prm.declare_entry("minimum_temperature", "20.0", Patterns::Double(0), "minimum_temperature");
        prm.declare_entry("maximum_temperature", "50.0", Patterns::Double(0), "maximum_temperature");
        prm.declare_entry("plateau_temperature", "0.1", Patterns::Double(0), "plateau_temperature");
    }
    prm.leave_subsection();

    prm.enter_subsection("Particle data");
    {
        prm.declare_entry("particle_density", "4000.", dealii::Patterns::Double(0.), "");
        prm.declare_entry("drag_coefficient", "0.47", dealii::Patterns::Double(0.), "");
        prm.declare_entry("particle_diameter", "0.01", dealii::Patterns::Double(0.), "");
        prm.declare_entry("particle_variance", "0.01", dealii::Patterns::Double(0.), "particle_variance");
        prm.declare_entry("inlet_particle_mass", "0.8", dealii::Patterns::Double(0.), "inlet_particle_mass");
        prm.declare_entry("inlet_duration", "4000.", dealii::Patterns::Double(0.), "inlet_duration");
        prm.declare_entry("fluid_density", "1.225", dealii::Patterns::Double(0.), "Kg/m^3");
        prm.declare_entry("gravity", "9.81", Patterns::Double(0.0), "Gravity");
        prm.declare_entry("initial_particle_velocity_x", "0", Patterns::Double(-100.0), "Initial Particle Velocity X");
        prm.declare_entry("initial_particle_velocity_y", "0", Patterns::Double(-100.0), "Initial Particle Velocity Y");
        prm.declare_entry("initial_particle_velocity_z", "0", Patterns::Double(-100.0), "Initial Particle Velocity Z");
        prm.declare_entry("initial_particle_temperature", "0", Patterns::Double(-100.0), "Initial Particle Temperature");
        prm.declare_entry("particle_specific_heat", "0", Patterns::Double(-100.0), "particle_specific_heat");
        prm.declare_entry("particle_absorptivity_rate", "0", Patterns::Double(-100.0), "particle_absorptivity_rate");
        prm.declare_entry("particle_convection_coefficient", "0", Patterns::Double(-100.0), "particle_convection_coefficient");
        prm.declare_entry("latent_heat", "0", Patterns::Double(-100.0), "latent_heat");
        prm.declare_entry("liquid_temperature", "0", Patterns::Double(-100.0), "liquid_temperature");
        prm.declare_entry("solid_temperature", "0", Patterns::Double(-100.0), "solid_temperature");
        prm.declare_entry("block_particles_on_boundary", "true", Patterns::Bool(), "Enable block particles on boundary: true|false");
        prm.declare_entry("parallelize_particles", "true", Patterns::Bool(), "parallelize_particles");
    }

    prm.leave_subsection();

    prm.enter_subsection("Laser data");
    {
        prm.declare_entry("laser_power", "500.", Patterns::Double(0.0), "");
        prm.declare_entry("laser_radius", "500.", Patterns::Double(0.0), "laser_radius");
        prm.declare_entry("laser_focal_plane", "500.", Patterns::Double(-100.0), "laser_focal_plane");
        prm.declare_entry("laser_angle", "500.", Patterns::Double(0.0), "laser_angle");
    }

    prm.leave_subsection();

    prm.enter_subsection("Solver parameters");
    {
        prm.declare_entry("algorithm","linear", Patterns::Selection("linear|linear_imex|newton|quasi_newton"), "Algorithm");
        prm.declare_entry("newton_tolerance", "1e-12", Patterns::Double(0), "newton_tolerance");
        prm.declare_entry("max_newton_iterations", "20", Patterns::Integer(1), "max_newton_iterations");
        prm.declare_entry("reinit_vel_preconditioner", "10", Patterns::Integer(1), "reinit_vel_preconditioner");
        prm.declare_entry("max_iterations", "1000", Patterns::Integer(1, 1000), " The maximal number of iterations GMRES must make. ");
        prm.declare_entry("eps", "1e-12", Patterns::Double(0.), " The stopping criterion. ");
        prm.declare_entry("Krylov_size", "30", Patterns::Integer(1), " The size of the Krylov subspace to be used. ");
        prm.declare_entry("off_diagonals", "60", Patterns::Integer(0), " The number of off-diagonal elements ILU must compute. ");
        prm.declare_entry("diag_strength", "0.01", Patterns::Double(0.), " Diagonal strengthening coefficient. ");
        prm.declare_entry("iPC_pressure_tollerance", "0.01", Patterns::Double(0.), " iPC_pressure_tollerance ");
        prm.declare_entry("iPC_max_pressure_iterations", "100", Patterns::Integer(0), " iPC_max_pressure_iterations ");
    }
    prm.leave_subsection();

    prm.enter_subsection("Output parameters");
    {
        prm.declare_entry("output", "10", Patterns::Integer(1), "Generate graphical output");
        prm.declare_entry("output_directory","output/", Patterns::Anything(), "output_directory");
        prm.declare_entry("output_filename","simulation_", Patterns::Anything(), "output_filename");
        prm.declare_entry("particles_filename","simulation_", Patterns::Anything(), "particles_filename");
        prm.declare_entry("write_to_file", "false", Patterns::Bool(), "write_to_file");
    }
    prm.leave_subsection();


    if (prm_filename.empty())
    {
        prm.print_parameters(std::cout, ParameterHandler::Text);
        AssertThrow(
                false, ExcMessage("please pass a .prm file as the first argument!"));
    }

    prm.parse_input(prm_filename);



    prm.enter_subsection("Simulation parameters");
    {
        simulation_time        = prm.get_double("simulation_time");
        time_step              = prm.get_double("time_step");
        theta                  = prm.get_double("theta");
        mesh_file              = prm.get("mesh_file");
        refinement_mesh_degree = prm.get_integer("refinement_mesh_degree");
    }
    prm.leave_subsection();

    prm.enter_subsection("FEM parameters");
    {
        fe_degree       = prm.get_integer("fe_degree");
    }
    prm.leave_subsection();

    prm.enter_subsection("Physical parameters");
    {
        inlet_manifold          = prm.get_integer("inlet_manifold");
        outlet_manifold         = prm.get_integer("outlet_manifold");
        walls_manifold          = prm.get_integer("walls_manifold");
        laser_manifold       = prm.get_integer("laser_manifold");
        substrate_manifold       = prm.get_integer("substrate_manifold");

        time_dependent_buondary     = prm.get_bool("time_dependent_buondary");

        l_channel               = prm.get_double("l_channel");
        h_channel               = prm.get_double("h_channel");
        substrate_location      = prm.get_double("substrate_location");
        outer_radius_coaxial_nozzle      = prm.get_double("outer_radius_coaxial_nozzle");
        inner_radius_coaxial_nozzle      = prm.get_double("inner_radius_coaxial_nozzle");
        top_point_mesh      = prm.get_double("top_point_mesh");
        channel_angle      = prm.get_double("channel_angle");
        box_radius      = prm.get_double("box_radius");

        inlet_velocity_x        = prm.get_double("inlet_velocity_x");
        inlet_velocity_y        = prm.get_double("inlet_velocity_y");
        inlet_velocity_z        = prm.get_double("inlet_velocity_z");
        inlet_type              = prm.get("inlet_type");
        reynolds_number         = prm.get_double("reynolds_number");
        strouhal_number         = prm.get_double("strouhal_number");
        prandtl_number          = prm.get_double("prandtl_number");
        minimum_temperature     = prm.get_double("minimum_temperature");
        maximum_temperature     = prm.get_double("maximum_temperature");
        plateau_temperature     = prm.get_double("plateau_temperature");

    }
    prm.leave_subsection();

    prm.enter_subsection("Particle data");
    {

        particle_density = prm.get_double("particle_density");
        inlet_particle_mass = prm.get_double("inlet_particle_mass");
        inlet_duration = prm.get_double("inlet_duration");
        particle_diameter = prm.get_double("particle_diameter");
        particle_variance = prm.get_double("particle_variance");
        drag_coefficient = prm.get_double("drag_coefficient");
        fluid_density = prm.get_double("fluid_density");
        gravity = prm.get_double("gravity");
        initial_particle_velocity_x = prm.get_double("initial_particle_velocity_x");
        initial_particle_velocity_y = prm.get_double("initial_particle_velocity_y");
        initial_particle_velocity_z = prm.get_double("initial_particle_velocity_z");
        initial_particle_temperature = prm.get_double("initial_particle_temperature");
        particle_specific_heat = prm.get_double("particle_specific_heat");
        particle_absorptivity_rate = prm.get_double("particle_absorptivity_rate");
        particle_convection_coefficient = prm.get_double("particle_convection_coefficient");
        latent_heat = prm.get_double("latent_heat");
        liquid_temperature = prm.get_double("liquid_temperature");
        solid_temperature = prm.get_double("solid_temperature");


        block_particles_on_boundary     = prm.get_bool("block_particles_on_boundary");
        parallelize_particles = prm.get_bool("parallelize_particles");
    }
    prm.leave_subsection();

    prm.enter_subsection("Laser data");
    {
        laser_power = prm.get_double("laser_power");
        laser_radius = prm.get_double("laser_radius");
        laser_focal_plane = prm.get_double("laser_focal_plane");
        laser_angle = prm.get_double("laser_angle");
    }
    prm.leave_subsection();

    prm.enter_subsection("Solver parameters");
    {
        algorithm       = prm.get("algorithm");
        newton_tolerance = prm.get_double("newton_tolerance");
        max_newton_iterations = prm.get_integer("max_newton_iterations");


        reinit_vel_preconditioner = prm.get_integer("reinit_vel_preconditioner");
        vel_max_iterations = prm.get_integer("max_iterations");
        vel_eps            = prm.get_double("eps");
        vel_Krylov_size    = prm.get_integer("Krylov_size");
        vel_off_diagonals  = prm.get_integer("off_diagonals");
        vel_diag_strength  = prm.get_double("diag_strength");
        iPC_pressure_tollerance  = prm.get_double("iPC_pressure_tollerance");
        iPC_max_pressure_iterations  = prm.get_integer("iPC_max_pressure_iterations");
    }
    prm.leave_subsection();

    prm.enter_subsection("Output parameters");
    {
        output           = prm.get_integer("output");
        output_directory = prm.get("output_directory");
        output_filename  = prm.get("output_filename");
        particles_filename  = prm.get("particles_filename");
        write_to_file    = prm.get_bool("write_to_file");
    }
    prm.leave_subsection();

    if (time_dependent_buondary == true && algorithm == "newton")
    {
        AssertThrow(false, ExcMessage(" 'time dependent buondary do not work with newton algorithm' "));
    }




}

void Settings::dump()
{
    deallog << std::endl;
    deallog << "SIMULATION PARAMETERS " << std::endl;
    deallog << "Time step: " << time_step << " [s]" << std::endl;
    deallog << "Simulation time: " << simulation_time << " [s]" <<  std::endl;
    deallog << "Refinement mesh degree: " << refinement_mesh_degree << std::endl;
    deallog << "Inclination angle: " << "25" << " [°]" << std::endl;
    deallog << std::endl;
    deallog << "FLUID PARAMETERS " << std::endl;
    deallog << "Inlet fluid velocity_x: " << inlet_velocity_x << " [m/s]" << std::endl;
    deallog << "Inlet fluid velocity_y: " << inlet_velocity_y << " [m/s]" << std::endl;
    deallog << "Inlet fluid velocity_z: " << inlet_velocity_z << " [m/s]" << std::endl;
    deallog << "Reynolds number: " << reynolds_number << std::endl;
    deallog << "Prandtl number: " << prandtl_number << std::endl;
    deallog << "Diffusion coefficient: " << 1/(reynolds_number*prandtl_number) << std::endl;
    deallog << std::endl;
}


