
#ifndef MULTIGRID_SETTINGS_H
#define MULTIGRID_SETTINGS_H

#include <iostream>
#include <fstream>
#include <string>

#include <deal.II/base/parameter_handler.h>
#include <deal.II/base/logstream.h>

class Settings
{

public:

    void get_parameters(const std::string &prm_filename);

    void dump();

    double                 reynolds_number;
    double                 strouhal_number;
    unsigned int           fe_degree;
    int                    output;
    std::string            mesh_file;
    double                 time_step;
    double                 simulation_time;
    double                 prandtl_number;
    double                 theta;
    bool                   write_to_file;
    int                    reinit_vel_preconditioner;
    int                    refinement_mesh_degree;
    std::string            output_directory;
    std::string            output_filename;
    std::string            particles_filename;
    double                 inlet_velocity_x;
    double                 inlet_velocity_y;
    double                 inlet_velocity_z;
    double                 minimum_temperature;
    double                 maximum_temperature;
    double                 plateau_temperature;
    double                 l_channel;
    double                 h_channel;
    double                 substrate_location;
    int                    inlet_manifold;
    int                    outlet_manifold;
    int                    walls_manifold;
    int                    laser_manifold;
    int                    substrate_manifold;
    std::string            inlet_type;
    std::string            algorithm;
    double                 newton_tolerance;
    int                    max_newton_iterations;
    unsigned int           vel_max_iterations;
    unsigned int           vel_Krylov_size;
    unsigned int           vel_off_diagonals;
    double                 vel_eps;
    double                 vel_diag_strength;
    double                 iPC_pressure_tollerance;
    int                    iPC_max_pressure_iterations;
    bool                   time_dependent_buondary;
    bool                   parallelize_particles;
    double                 latent_heat;
    double                 liquid_temperature;
    double                 solid_temperature;
    double                 outer_radius_coaxial_nozzle;
    double                 inner_radius_coaxial_nozzle;
    double                 top_point_mesh;
    double                 channel_angle;
    double                 box_radius;

    /**Particles variables**/

    double particle_density;
    double particle_diameter;
    double particle_variance;
    double inlet_particle_mass;
    double inlet_duration;
    double drag_coefficient;
    double fluid_density;
    double gravity;
    double initial_particle_velocity_x;
    double initial_particle_velocity_y;
    double initial_particle_velocity_z;
    double initial_particle_temperature;
    double particle_specific_heat;
    double particle_absorptivity_rate;
    double particle_convection_coefficient;
    bool   block_particles_on_boundary;
    /**********************/

    /**Particles variables**/
    double laser_power;
    double laser_radius;
    double laser_focal_plane;
    double laser_angle;
    /**********************/

};





#endif //MULTIGRID_SETTINGS_H
