
#include "temperature.h"


Temperature::Temperature(const Settings &settings, Triangulation<dim> &triangulation)
        : triangulation(triangulation)
        , dof_handler_temperature(triangulation)
        , fe_temperature(settings.fe_degree)
        , quadrature_temperature(settings.fe_degree+1)
        , mapping_temperature(settings.fe_degree)
        , settings(settings)
        , diffusion_coefficient(1/(settings.reynolds_number*settings.prandtl_number))
{
    temperature_imposed.plateau_time = settings.plateau_temperature;
    temperature_imposed.min_t = settings.minimum_temperature;
    temperature_imposed.max_t = settings.maximum_temperature;
    temperature_imposed.laser_radius = settings.laser_radius;
    temperature_imposed.substrate_location = settings.substrate_location;

    temperature_inlet.temp_inlet = settings.minimum_temperature;
    temperature_initial.temp_init = settings.maximum_temperature;

    temperature_right_hand_side.laser_power = settings.laser_power;
    temperature_right_hand_side.substrate_location = settings.substrate_location;
    temperature_right_hand_side.laser_radius = settings.laser_radius;




}


void Temperature::setup_system()
{

    dof_handler_temperature.distribute_dofs(fe_temperature);
    DoFRenumbering::boost::Cuthill_McKee(dof_handler_temperature);

    constraints_temperature.clear();
    DoFTools::make_hanging_node_constraints(dof_handler_temperature, constraints_temperature);
//    VectorTools::interpolate_boundary_values(
//            mapping_temperature, dof_handler_temperature, 0, BoundaryValues<dim>(), constraints_temperature);
//    VectorTools::interpolate_boundary_values(
//            mapping_temperature, dof_handler_temperature, 1, BoundaryValues<dim>(), constraints_temperature);
    constraints_temperature.close();

    DynamicSparsityPattern dspt(dof_handler_temperature.n_dofs());
    DoFTools::make_sparsity_pattern(dof_handler_temperature,
                                    dspt,
                                    constraints_temperature,
            /*keep_constrained_dofs = */ false);

    sparsity_pattern_temperature.copy_from(dspt);
    system_matrix.reinit(sparsity_pattern_temperature);

    temperature.reinit(dof_handler_temperature.n_dofs());
    old_temperature.reinit(dof_handler_temperature.n_dofs());
    old_old_temperature.reinit(dof_handler_temperature.n_dofs());
    system_rhs.reinit(dof_handler_temperature.n_dofs());

    alpha_punto_uno.reinit(dof_handler_temperature.n_dofs());
    alpha_punto_zero.reinit(dof_handler_temperature.n_dofs());




    // Initialize solution vectors
    VectorTools::interpolate(dof_handler_temperature, temperature_initial, temperature);
    VectorTools::interpolate(dof_handler_temperature, temperature_initial, old_temperature);
    VectorTools::interpolate(dof_handler_temperature, temperature_initial, old_old_temperature);


    //VectorTools::project(dof_handler_temperature, constraints_temperature, quadrature_temperature, laser_power_function, laser_radiation);

}


