#ifndef MULTIGRID_TEMPERATURE_H
#define MULTIGRID_TEMPERATURE_H

#include "../general.h"
#include "../settings.h"
#include "../linearized_ns/navierstokes.h"
#include "temperature_bcs.h"
#include "temperature_rhs.h"
#include "../newton_ns/newton.h"


class Temperature
{
public:
    Temperature(const Settings &settings, Triangulation<dim> &triangulation);

    void setup_system();

    void assemble_system_and_multigrid_linear_ns(const NavierStokes &ns);
    void assemble_system_and_multigrid_nonlinear_ns(const NavierStokesNewton &ns);

    void solve_linear_ns(const NavierStokes &ns);
    void solve_nonlinear_ns(const NavierStokesNewton &ns);

    double compute_stabilization_delta(const double         hk,
                                       const double         eps,
                                       const Tensor<1, dim> dir,
                                       const double         pk) const;

    void output_results(DataOut<dim> &data_out) const;

    Triangulation<dim> &triangulation;
    DoFHandler<dim>    dof_handler_temperature;

    const FE_Q<dim>     fe_temperature;
    QGauss<dim>         quadrature_temperature;
    const MappingQ<dim> mapping_temperature;

    AffineConstraints<double> constraints_temperature;

    SparsityPattern      sparsity_pattern_temperature;
    SparseMatrix<double> system_matrix;

    Vector<double> temperature;
    Vector<double> old_temperature;
    Vector<double> old_old_temperature;
    Vector<double> system_rhs;

    Vector<double> alpha_punto_uno;
    Vector<double> alpha_punto_zero;

    const Settings &settings;

    const double diffusion_coefficient;

    TemperatureImposed temperature_imposed;
    TemperatureInlet temperature_inlet;
    TemperatureInitial temperature_initial;
    TemperatureRightHandSide temperature_right_hand_side;



};


#endif //MULTIGRID_TEMPERATURE_H
