
#include "temperature.h"
#include "temperature_worker.h"


using IteratorTuple = std::tuple<typename DoFHandler<dim>::active_cell_iterator,
        typename DoFHandler<dim>::active_cell_iterator>;
using IteratorPair = SynchronousIterators<IteratorTuple>;


template <class IteratorType>
void assemble_cell_nonlinear_ns(const IteratorType &cell,
                   T_ScratchData_nonlinear_ns &scratch_data,
                   T_CopyData_nonlinear_ns &copy_data)
{
    auto vel_cell = std::get<0>(*cell);
    auto temp_cell = std::get<1>(*cell);

    copy_data.level = temp_cell->level();

    unsigned int n_q_points(scratch_data.fe_values_temperature.get_quadrature().size());
    unsigned int temp_dpc(scratch_data.fe_values_temperature.get_fe().dofs_per_cell);

    copy_data.dof_per_cell_temperature = temp_dpc;
    copy_data.cell_matrix.reinit(temp_dpc, temp_dpc);
    if (temp_cell->is_level_cell() == false)
        copy_data.cell_rhs.reinit(temp_dpc);

    copy_data.local_dof_indices_temperature.resize(temp_dpc);

    temp_cell->get_active_or_mg_dof_indices(copy_data.local_dof_indices_temperature);
//    temp_cell->get_dof_indices(copy_data.local_dof_indices_temperature);


    scratch_data.fe_newton.reinit(vel_cell);
    scratch_data.fe_values_temperature.reinit(temp_cell);

    std::vector<double> old_temperature_values(n_q_points);
    std::vector<Tensor<1, dim>> old_temperature_values_grad(n_q_points);
    std::vector<double> old_temperature_values_laplacian(n_q_points);
    std::vector<double> alpha_punto_uno_values(n_q_points);
    std::vector<double> old_old_temperature_values(n_q_points);

    std::vector<Tensor < 1, dim>> old_velocity_values(n_q_points);
    std::vector<Tensor < 1, dim>> old_old_velocity_values(n_q_points);

    //std::vector<Vector<double >> old_v(n_q_points, Vector<double>(dim));
    //std::vector<Vector<double >> old_old_v(n_q_points, Vector<double>(dim));


    scratch_data.fe_values_temperature.get_function_values(scratch_data.temperature.old_temperature, old_temperature_values);
    scratch_data.fe_values_temperature.get_function_gradients(scratch_data.temperature.old_temperature, old_temperature_values_grad);
    scratch_data.fe_values_temperature.get_function_laplacians(scratch_data.temperature.old_temperature, old_temperature_values_laplacian);
    scratch_data.fe_values_temperature.get_function_values(scratch_data.temperature.alpha_punto_uno, alpha_punto_uno_values);
    scratch_data.fe_values_temperature.get_function_values(scratch_data.temperature.old_old_temperature, old_old_temperature_values);

    const FEValuesExtractors::Vector velocities(0);

    scratch_data.fe_newton[velocities].get_function_values(scratch_data.ns.present_solution, old_velocity_values);
    scratch_data.fe_newton[velocities].get_function_values(scratch_data.ns.present_solution_minus_1, old_old_velocity_values);

    /*
    for (unsigned int q = 0; q < n_q_points; ++q) {
        for (unsigned int d = 0; d < dim; ++d) {
            old_velocity_values[q][d] = old_v[q](d);
            old_old_velocity_values[q][d] = old_old_v[q](d);
        }
    }
    */

    copy_data.cell_matrix = 0;
    copy_data.cell_rhs = 0;

    Tensor<1, dim> advection_direction;
    Tensor<1, dim> old_advection_direction;
    for (unsigned int q_point = 0; q_point < n_q_points; ++q_point) {

        for (unsigned int d = 0; d < dim; ++d) {
            advection_direction[d] = old_velocity_values[q_point][d];
            old_advection_direction[d] = old_old_velocity_values[q_point][d];
        }

        const double delta = scratch_data.temperature.compute_stabilization_delta(temp_cell->diameter(),
                                                         scratch_data.temperature.diffusion_coefficient,
                                                         advection_direction,
                                                         scratch_data.temperature.settings.fe_degree);


        for (unsigned int i = 0; i < temp_dpc; ++i) {
            for (unsigned int j = 0; j < temp_dpc; ++j) {

                copy_data.cell_matrix(i, j) +=
                        // M
                        scratch_data.temperature.settings.strouhal_number *
                        (3.0/2.0)*
                        (scratch_data.fe_values_temperature.shape_value(i, q_point) *
                         scratch_data.fe_values_temperature.shape_value(j, q_point) *
                         scratch_data.fe_values_temperature.JxW(q_point))

                        +

                        // M + SUPG
                        scratch_data.temperature.settings.strouhal_number *
                        (3.0/2.0)*
                        (scratch_data.fe_values_temperature.shape_value(j, q_point)) *
                        delta *
                        (advection_direction *
                         scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                         scratch_data.fe_values_temperature.JxW(q_point))

                        +

                        // Kc
                        scratch_data.temperature.settings.theta *
                        (scratch_data.temperature.settings.time_step) *
                        (scratch_data.temperature.diffusion_coefficient *
                         scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                         scratch_data.fe_values_temperature.shape_grad(j, q_point) *
                         scratch_data.fe_values_temperature.JxW(q_point))

                        +

                        // Kc + SUPG
                        scratch_data.temperature.settings.theta *
                        (scratch_data.temperature.settings.time_step) *
                        (-scratch_data.temperature.diffusion_coefficient) *
                        trace(scratch_data.fe_values_temperature.shape_hessian(j, q_point)) *
                        delta *
                        (advection_direction *
                         scratch_data.fe_values_temperature.shape_grad(i, q_point)) *
                        scratch_data.fe_values_temperature.JxW(q_point)

                        +

                        // K(v_1)
                        scratch_data.temperature.settings.theta *
                        (scratch_data.temperature.settings.time_step) *
                        (scratch_data.fe_values_temperature.shape_value(i, q_point) *
                         (advection_direction *
                          scratch_data.fe_values_temperature.shape_grad(j, q_point)) *
                         scratch_data.fe_values_temperature.JxW(q_point))

                        +

                        // K(v_1) + SUPG
                        scratch_data.temperature.settings.theta *
                        (scratch_data.temperature.settings.time_step) *
                        delta *
                        (advection_direction *
                         scratch_data.fe_values_temperature.shape_grad(j, q_point)) *
                        (advection_direction *
                         scratch_data.fe_values_temperature.shape_grad(i, q_point)) *
                        scratch_data.fe_values_temperature.JxW(q_point);


                if (temp_cell->is_level_cell() == false) {
                    copy_data.cell_rhs(i) +=
                            // M
                            scratch_data.temperature.settings.strouhal_number *
                            2.0*
                            (scratch_data.fe_values_temperature.shape_value(i, q_point) *
                             scratch_data.fe_values_temperature.shape_value(j, q_point) *
                             old_temperature_values[q_point] *
                             scratch_data.fe_values_temperature.JxW(q_point))

                            +

                            // M + SUPG
                            scratch_data.temperature.settings.strouhal_number *
                            2.0*
                            (scratch_data.fe_values_temperature.shape_value(j, q_point)) *
                            old_temperature_values[q_point] *
                            delta *
                            (advection_direction *
                             scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                             scratch_data.fe_values_temperature.JxW(q_point))

                            +

                            // M(BDF2)
                            scratch_data.temperature.settings.strouhal_number *
                            (-1.0/2.0)*
                            (scratch_data.fe_values_temperature.shape_value(i, q_point) *
                             scratch_data.fe_values_temperature.shape_value(j, q_point) *
                             old_old_temperature_values[q_point] *
                             scratch_data.fe_values_temperature.JxW(q_point))

                            +

                            // M + SUPG(BDF2)
                            scratch_data.temperature.settings.strouhal_number *
                            (-1.0/2.0)*
                            (scratch_data.fe_values_temperature.shape_value(j, q_point)) *
                            old_old_temperature_values[q_point] *
                            delta *
                            (advection_direction *
                             scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                             scratch_data.fe_values_temperature.JxW(q_point))


                    -

                            (1 - scratch_data.temperature.settings.theta) *
                            (
                                    // Kc
                                    (scratch_data.temperature.settings.time_step) *
                                    scratch_data.temperature.diffusion_coefficient *
                                    scratch_data.fe_values_temperature.shape_value(j, q_point) *
                                    old_temperature_values_grad[q_point] *
                                    (scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                                     scratch_data.fe_values_temperature.JxW(q_point))

                                    +

                                    // Kc + SUPG
                                    (scratch_data.temperature.settings.time_step) *
                                    -scratch_data.temperature.diffusion_coefficient *
                                    scratch_data.fe_values_temperature.shape_value(j, q_point) *
                                    old_temperature_values_laplacian[q_point] *
                                    (delta *
                                     advection_direction *
                                     scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                                     scratch_data.fe_values_temperature.JxW(q_point))

                                    +

                                    // K(v_0)
                                    (scratch_data.temperature.settings.time_step) *
                                    advection_direction *
                                    old_temperature_values_grad[q_point] *
                                    scratch_data.fe_values_temperature.shape_value(j, q_point) *
                                    scratch_data.fe_values_temperature.shape_value(i, q_point) *
                                    scratch_data.fe_values_temperature.JxW(q_point)

                                    +

                                    // K(v_0) + SUPG
                                    (scratch_data.temperature.settings.time_step) *
                                    advection_direction *
                                    scratch_data.fe_values_temperature.shape_value(j, q_point) *
                                    old_temperature_values_grad[q_point] *
                                    (delta *
                                     advection_direction *
                                     scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                                     scratch_data.fe_values_temperature.JxW(q_point))
                            )

                            +

                            //M(u_1)-M(u_0)
                            scratch_data.temperature.settings.time_step *
                            (1 - scratch_data.temperature.settings.theta) *
                            (delta *
                             (advection_direction - old_advection_direction) *
                             scratch_data.fe_values_temperature.shape_grad(i, q_point) *
                             scratch_data.fe_values_temperature.shape_value(j, q_point) *
                             alpha_punto_uno_values[q_point] *
                             scratch_data.fe_values_temperature.JxW(q_point));
                }
            }
        }

    }
}



void Temperature::assemble_system_and_multigrid_nonlinear_ns(const NavierStokesNewton &ns)
{
//    system_matrix.reinit(sparsity_pattern_temperature);
//    system_rhs.reinit(dof_handler_temperature.n_dofs());

    system_matrix = 0;
    system_rhs = 0;

    unsigned int n;
    for (n = 0; n<alpha_punto_uno.size(); ++n)
    {
        alpha_punto_uno[n] = (1/(settings.theta*settings.time_step))*
                             (old_temperature[n]-old_old_temperature[n])
                             -((1-settings.theta)/settings.theta)*alpha_punto_zero[n];
    }

    auto cell_worker_active = [&](const IteratorPair &cell,
                                  T_ScratchData_nonlinear_ns &scratch_data,
                                  T_CopyData_nonlinear_ns &copy_data) {
        assemble_cell_nonlinear_ns(cell, scratch_data, copy_data);
    };

    auto copier_active = [&](const T_CopyData_nonlinear_ns &copy_data) {
        constraints_temperature.distribute_local_to_global(copy_data.cell_matrix,
                                                           copy_data.cell_rhs,
                                                           copy_data.local_dof_indices_temperature,
                                                           system_matrix,
                                                           system_rhs);
    };

    WorkStream::run(IteratorPair(IteratorTuple(ns.dof_handler.begin_active(),
                                               dof_handler_temperature.begin_active())),
                    IteratorPair(IteratorTuple(ns.dof_handler.end(),
                                               dof_handler_temperature.end())),
                    cell_worker_active,
                    copier_active,
                    T_ScratchData_nonlinear_ns(*this, ns),
                    T_CopyData_nonlinear_ns(*this));
}

