
#include "temperature_bcs.h"

TemperatureInitial::TemperatureInitial()
        : Function<dim>(1) {}

double TemperatureInitial::value(const Point<dim> &p,
                               const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);

    return temp_init;
}
void TemperatureInitial::value_list(const std::vector<Point<dim>> &points,
                                  std::vector<double> &values,
                                  const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);
    const unsigned int n_points = points.size();
    Assert(values.size() == n_points,
           ExcDimensionMismatch(values.size(), n_points));
    for (unsigned int i = 0; i < n_points; ++i)
        values[i] = TemperatureInitial::value(points[i]);
}



TemperatureInlet::TemperatureInlet()
        : Function<dim>(1) {}
double TemperatureInlet::value(const Point<dim> &/*p*/,
                               const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);

    return temp_inlet;
}
void TemperatureInlet::value_list(const std::vector<Point<dim>> &points,
                                  std::vector<double> &values,
                                  const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);
    const unsigned int n_points = points.size();
    Assert(values.size() == n_points,
           ExcDimensionMismatch(values.size(), n_points));
    for (unsigned int i = 0; i < n_points; ++i)
        values[i] = TemperatureInlet::value(points[i]);
}



TemperatureImposed::TemperatureImposed()
        : Function<dim>(1)
{}

double TemperatureImposed::value(const Point<dim> &p,
                                 const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);


    double radius = sqrt(pow(std::fabs(p(0)),2) + pow(std::fabs(p(1)),2));

    if(radius < laser_radius - 1e-12)
        return max_t;
    else
        return min_t;


    return max_t;

/*
    if(time < plateau_time)
        return min_t + (time/plateau_time)*(max_t-min_t);
    else
        return max_t;
*/

}
void TemperatureImposed::value_list(const std::vector<Point<dim>> &points,
                                    std::vector<double> &values,
                                    const unsigned int component) const {
    (void) component;
    AssertIndexRange(component, 1);
    const unsigned int n_points = points.size();
    Assert(values.size() == n_points,
           ExcDimensionMismatch(values.size(), n_points));
    for (unsigned int i = 0; i < n_points; ++i)
        values[i] = TemperatureImposed::value(points[i]);
}
