#ifndef MULTIGRID_TEMPERATURE_BCS_H
#define MULTIGRID_TEMPERATURE_BCS_H

#include <deal.II/base/function.h>
#include <deal.II/lac/vector.h>

#include "../general.h"


class TemperatureInitial : public Function<dim> {
public:

    double temp_init;

    TemperatureInitial();

    virtual double value(const Point<dim> &p,
                         const unsigned int component = 0) const override;

    virtual void value_list(const std::vector<Point<dim>> &points,
                            std::vector<double> &values,
                            const unsigned int component = 0) const override;
};




class TemperatureInlet : public Function<dim> {
public:

    double temp_inlet;

    TemperatureInlet();

    virtual double value(const Point<dim> &p,
                         const unsigned int component = 0) const override;

    virtual void value_list(const std::vector<Point<dim>> &points,
                            std::vector<double> &values,
                            const unsigned int component = 0) const override;
};

class TemperatureImposed : public Function<dim> {
public:

    double time, plateau_time, min_t, max_t;
    double laser_radius;
    double substrate_location;

    TemperatureImposed();

    virtual double value(const Point<dim> &p,
                         const unsigned int component = 0) const override;

    virtual void value_list(const std::vector<Point<dim>> &points,
                            std::vector<double> &values,
                            const unsigned int component = 0) const override;
};




#endif //MULTIGRID_TEMPERATURE_BCS_H
