
#include "temperature.h"

void Temperature::solve_linear_ns(const NavierStokes &ns)
{
    statistics->enter_subsection("Temperature Assembly");

    assemble_system_and_multigrid_linear_ns(ns);



    //Apply bounday values

    std::map<types::global_dof_index, double> temperature_boundary_values;
    temperature_boundary_values.clear();


    for (const int &boundary_id : triangulation.get_boundary_ids())
    {

            if (boundary_id == settings.inlet_manifold)
            {
                VectorTools::interpolate_boundary_values(dof_handler_temperature,
                                                         boundary_id,
                                                         temperature_inlet,
                                                         temperature_boundary_values);
            }
/*
            else if (boundary_id == settings.substrate_manifold)
            {
                VectorTools::interpolate_boundary_values(dof_handler_temperature,
                                                         boundary_id,
                                                         temperature_imposed,
                                                         temperature_boundary_values);
            }

            else if (boundary_id == settings.laser_manifold)
            {

                VectorTools::interpolate_boundary_values(dof_handler_temperature,
                                                         boundary_id,
                                                         temperature_imposed,
                                                         temperature_boundary_values);
            }
            */
    }
    




//    VectorTools::interpolate_boundary_values(temperature_dof_handler, 0, BoundaryValues(), temperature_boundary_values);
    MatrixTools::apply_boundary_values(temperature_boundary_values, system_matrix, temperature, system_rhs);

    statistics->leave_subsection();

    statistics->enter_subsection("Temperature Solver");

    const unsigned int max_iters = 1000;
    const double solve_tolerance = 1e-8 * system_rhs.l2_norm();
    SolverControl solver_control(max_iters, solve_tolerance, true, true);
    solver_control.enable_history_data();
    SolverGMRES<> solver(solver_control,
                         SolverGMRES<>::AdditionalData(50, true));
    SparseILU<double> preconditioner;
    preconditioner.initialize(system_matrix);
    deallog << "Solving Temperature with GMRES Standard to tol " << solve_tolerance << std::endl;

    Timer time;
    time.start();
    solver.solve(system_matrix, temperature, system_rhs, preconditioner);
    time.stop();

    deallog << " converged in " << solver_control.last_step()
            << " iterations"
            << " in " << time.last_wall_time() << " seconds "
            << "residual: " << solver_control.last_value() << std::endl;

    constraints_temperature.distribute(temperature);

    statistics->leave_subsection();

    statistics->enter_subsection("Temperature Update Old Values");

    double min_temperature = temperature(0),
            max_temperature = temperature(0);
    for (unsigned int i = 0; i < temperature.size(); ++i) {
        min_temperature = std::min<double>(min_temperature, temperature(i));
        max_temperature = std::max<double>(max_temperature, temperature(i));
    }
    deallog << "Temperature range: " << min_temperature << ' '
              << max_temperature << std::endl;

    old_old_temperature = old_temperature;
    old_temperature = temperature;

    statistics->leave_subsection();

}