
#include "temperature_worker.h"


T_ScratchData::T_ScratchData(const Temperature  &temperature,
                             const NavierStokes &ns,
                             const UpdateFlags  update_flags)
        :
        temperature(temperature),
        ns(ns),
        update_scratch(update_flags),
        fe_values_temperature(temperature.fe_temperature,
                           temperature.quadrature_temperature,
                           update_scratch),
        fe_values_velocity(ns.fe_velocity,
                           temperature.quadrature_temperature,
                           update_scratch)
{}

T_ScratchData::T_ScratchData(const T_ScratchData &scratch_data)
        :
        temperature(scratch_data.temperature),
        ns(scratch_data.ns),
        update_scratch(scratch_data.update_scratch),
        fe_values_temperature(scratch_data.fe_values_temperature.get_fe(),
                           scratch_data.fe_values_temperature.get_quadrature(),
                           scratch_data.update_scratch),
        fe_values_velocity(scratch_data.fe_values_velocity.get_fe(),
                           scratch_data.fe_values_velocity.get_quadrature(),
                           scratch_data.update_scratch)
{}


T_ScratchData_nonlinear_ns::T_ScratchData_nonlinear_ns(const Temperature  &temperature,
                             const NavierStokesNewton &ns,
                             const UpdateFlags  update_flags)
        :
        temperature(temperature),
        ns(ns),
        update_scratch(update_flags),
        fe_values_temperature(temperature.fe_temperature,
                              temperature.quadrature_temperature,
                              update_scratch),
        fe_newton(ns.fe,
                temperature.quadrature_temperature,
                update_scratch)
{}

T_ScratchData_nonlinear_ns::T_ScratchData_nonlinear_ns(const T_ScratchData_nonlinear_ns &scratch_data)
        :
        temperature(scratch_data.temperature),
        ns(scratch_data.ns),
        update_scratch(scratch_data.update_scratch),
        fe_values_temperature(scratch_data.fe_values_temperature.get_fe(),
                              scratch_data.fe_values_temperature.get_quadrature(),
                              scratch_data.update_scratch),
        fe_newton(scratch_data.fe_newton.get_fe(),
                           scratch_data.fe_newton.get_quadrature(),
                           scratch_data.update_scratch)
{}



