

#ifndef MULTIGRID_TEMPERATURE_WORKER_H
#define MULTIGRID_TEMPERATURE_WORKER_H

#include <deal.II/fe/fe_q.h>
#include <deal.II/fe/mapping_q.h>
#include <deal.II/fe/fe_values.h>

#include <deal.II/lac/vector.h>
#include <deal.II/lac/full_matrix.h>

#include "../general.h"
#include "temperature.h"

using namespace dealii;

class T_ScratchData
{
public:

    T_ScratchData(const Temperature &temperature,
                  const NavierStokes &ns,
                  const UpdateFlags         update_flags = update_values |
                                                            update_gradients |
                                                            update_hessians |
                                                            update_quadrature_points |
                                                            update_JxW_values);

    T_ScratchData(const T_ScratchData &scratch_data);

    const Temperature &temperature;
    const NavierStokes &ns;
    UpdateFlags update_scratch;
    FEValues<dim> fe_values_temperature;
    FEValues<dim> fe_values_velocity;

};


class T_CopyData
{
public:
    T_CopyData(const Temperature &temperature)
            : temperature(&temperature)
    {};

    int level;
    int dof_per_cell_temperature;

    FullMatrix<double>                   cell_matrix;
    Vector<double>                       cell_rhs;
    std::vector<types::global_dof_index> local_dof_indices_temperature;

    const Temperature *temperature;



};

class T_ScratchData_nonlinear_ns
{
public:

    T_ScratchData_nonlinear_ns(const Temperature &temperature,
                  const NavierStokesNewton &ns,
                  const UpdateFlags         update_flags = update_values |
                                                           update_gradients |
                                                           update_hessians |
                                                           update_quadrature_points |
                                                           update_JxW_values);

    T_ScratchData_nonlinear_ns(const T_ScratchData_nonlinear_ns &scratch_data);

    const Temperature &temperature;
    const NavierStokesNewton &ns;
    UpdateFlags update_scratch;
    FEValues<dim> fe_values_temperature;
    FEValues<dim> fe_newton;


};


class T_CopyData_nonlinear_ns
{
public:
    T_CopyData_nonlinear_ns(const Temperature &temperature)
            : temperature(&temperature)
    {};

    int level;
    int dof_per_cell_temperature;

    FullMatrix<double>                   cell_matrix;
    Vector<double>                       cell_rhs;
    std::vector<types::global_dof_index> local_dof_indices_temperature;

    const Temperature *temperature;
};

#endif //MULTIGRID_TEMPERATURE_WORKER_H
